using FluentValidation;
using ExpenseTracker.Domain.Enums;

namespace ExpenseTracker.Application.Accounts.Commands.Create;

public class CreateAccountCommandValidator : AbstractValidator<CreateAccountCommand>
{
    public CreateAccountCommandValidator()
    {
        RuleFor(e => e.Name)
            .NotEmpty();

        RuleFor(e => e.InitialBalance)
            .NotNull();

        RuleFor(e => e.Currency)
            .Must(c => Currency.FromName(c) is not null)
            .WithMessage(c => $"'{nameof(c.Currency)}' must be one of the following: '{String.Join("', ", Currency.Enumerations.Values.Select(v => v.Name))}'.");
    }
}
