using AutoMapper;
using MediatR;
using ExpenseTracker.Application.Common.Interfaces.Repositories;
using ExpenseTracker.Application.Common.Interfaces.Services;
using ExpenseTracker.Domain.Enums;

namespace ExpenseTracker.Application.Accounts.Queries.Charts.GetExpenseByCategoryPieChart;

public class GetExpenseByCategoryPieChartQueryHandler : IRequestHandler<GetExpenseByCategoryPieChartQuery, Dictionary<Category, double>>
{
    private readonly IMapper _mapper;
    private readonly IAccountRepository _accountRepository;
    private readonly ITransactionRepository _transactionRepository;
    private readonly ISessionUserService _sessionUserService;

    public GetExpenseByCategoryPieChartQueryHandler(
        IMapper mapper,
        IAccountRepository repository,
        ITransactionRepository transactionRepository,
        ISessionUserService sessionUserService)
    {
        _mapper = mapper;
        _accountRepository = repository;
        _sessionUserService = sessionUserService;
        _transactionRepository = transactionRepository;
    }

    public async Task<Dictionary<Category, double>> Handle(GetExpenseByCategoryPieChartQuery request, CancellationToken cancellationToken)
    {
        var entities = _transactionRepository.Queryable
            .Where(e =>
                e.AccountId == request.AccountId &&
                e.Amount < 0)
            .ToList()
            .Where(e =>
                DateOnly.FromDateTime(e.Time.DateTime) >= request.FromDate &&
                DateOnly.FromDateTime(e.Time.DateTime) < request.ToDate)
            .OrderBy(e => e.Time);

        var groupsByCategory = entities.GroupBy(e => e.Category, t => t.Amount);

        var pieChart = new Dictionary<Category, double>();

        var totalAmount = groupsByCategory.Aggregate(0.0, (a, x) => a + x.Aggregate(0.0, (b, y) => b + y));

        foreach (var group in groupsByCategory)
        {
            var category = group.Key;
            var amount = group.Aggregate(0.0, (a, x) => a + x);
            var percent = Double.Round(amount / totalAmount, 4);

            pieChart.Add(category, percent);
        }

        return pieChart;
    }
}
