using MediatR.Behaviors.Authorization;
using ExpenseTracker.Application.Common.Authorization;
using ExpenseTracker.Application.Common.Interfaces.Services;
using ExpenseTracker.Application.Common.Models;

namespace ExpenseTracker.Application.Users.Queries.GetWithPagination;

public class GetUsersWithPaginationQueryAuthorizer : AbstractRequestAuthorizer<GetUsersWithPaginationQuery>
{
    private readonly ISessionUserService _sessionUserService;

    public GetUsersWithPaginationQueryAuthorizer(ISessionUserService currentUserService)
    {
        _sessionUserService = currentUserService;
    }

    public override void BuildPolicy(GetUsersWithPaginationQuery request)
    {
        UseRequirement(new MustBeAuthenticatedRequirement
        {
            IsAuthenticated = _sessionUserService.IsAuthenticated
        });

        UseRequirement(new MustBeInRolesRequirement
        {
            UserRoles = _sessionUserService.Roles,
            RequiredRoles = new[] { IdentityRoles.Administrator.ToString() }
        });
    }
}
